<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">    
<xsl:output method="text"/>
    <xsl:template match="/">
        <xsl:apply-templates select="problem"/>
    </xsl:template>
    
    <xsl:template match="problem">
        <xsl:apply-templates select="trs/signature"/>
        <xsl:apply-templates select="strategy"/>
        <xsl:apply-templates select="trs/rules"/>
    </xsl:template>
    
    <xsl:template match="rules">
      <xsl:text>[
</xsl:text>
      <xsl:for-each select="rule">
        <xsl:apply-templates select="."/>
        <xsl:if test="position() != last()">
          <xsl:text>,
</xsl:text>
        </xsl:if>
      </xsl:for-each>  
      <xsl:apply-templates select="//relrules/rule" mode="relative"/>
      <xsl:text>
]
</xsl:text>
    </xsl:template>
    
    <xsl:template match="signature">
      <xsl:if test="//theory">
        <xsl:message terminate="yes">
          <xsl:text>(THEORY </xsl:text>
          <xsl:apply-templates select="//theory[not(.=preceding::theory)]"/>
          <xsl:text>) is not supported in CeTA format!</xsl:text>
        </xsl:message>
      </xsl:if>    
    </xsl:template>
    
    <xsl:template match="strategy">
      <xsl:choose>
        <xsl:when test="//replacementmap">
          <xsl:message terminate="yes">
            <xsl:text>(STRATEGY CONTEXTSENSITIVE</xsl:text>
            <xsl:apply-templates select="//replacementmap"/>
            <xsl:text>) is not supported in CeTA format!</xsl:text>
          </xsl:message>
        </xsl:when>
        <xsl:when test=".!='FULL'">
          <xsl:message terminate="yes">
            <xsl:text>(STRATEGY </xsl:text>
            <xsl:value-of select="."/>
            <xsl:text>) is not supported in CeTA format!</xsl:text>
          </xsl:message>
        </xsl:when>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="rule">
      <xsl:text>(</xsl:text>
      <xsl:apply-templates select="lhs"/>
      <xsl:text>,
   </xsl:text>
       <xsl:apply-templates select="rhs"/>
       <xsl:text>)</xsl:text>
      <xsl:if test="conditions">
        <xsl:message terminate="yes">
          <xsl:text>Conditional rules are not supported in CeTA format!</xsl:text>
        </xsl:message>
      </xsl:if>
    </xsl:template>
    
    <xsl:template match="rule" mode="relative">
      <xsl:message terminate="yes">
        <xsl:text>Relative rules are not supported in CeTA format!</xsl:text>
      </xsl:message>
    </xsl:template>    
    
    <xsl:template match="funapp">
      <xsl:text>Fun "</xsl:text>
      <xsl:call-template name="escape">
        <xsl:with-param name="outputString" select="name"/>
      </xsl:call-template>
      <xsl:text>" [</xsl:text>
      <xsl:for-each select="arg">
        <xsl:apply-templates select="."/>
        <xsl:if test="position() != last()">
          <xsl:text>, </xsl:text>
        </xsl:if>
      </xsl:for-each>
      <xsl:text>]</xsl:text>
    </xsl:template>
    
    <xsl:template match="arg">
        <xsl:apply-templates select="funapp"/>
        <xsl:apply-templates select="var"/>
    </xsl:template>
    
    <xsl:template match="var">
      <xsl:text>Var "</xsl:text>
      <xsl:call-template name="escape">
        <xsl:with-param name="outputString" select="."/>
      </xsl:call-template>
      <xsl:text>"</xsl:text>
    </xsl:template>

    <xsl:template name="escape">
      <xsl:param name="outputString"/>
      <xsl:call-template name="globalReplace">
        <xsl:with-param name="outputString" select="$outputString"/>
        <xsl:with-param name="target" select="'\'"/>
        <xsl:with-param name="replacement" select="'\\'"/>
      </xsl:call-template>
    </xsl:template>

    <xsl:template name="globalReplace">
      <xsl:param name="outputString"/>
      <xsl:param name="target"/>
      <xsl:param name="replacement"/>
      <xsl:choose>
        <xsl:when test="contains($outputString,$target)">          
          <xsl:value-of select="concat(substring-before($outputString,$target),$replacement)"/>
          <xsl:call-template name="globalReplace">
            <xsl:with-param name="outputString" select="substring-after($outputString,$target)"/>
            <xsl:with-param name="target" select="$target"/>
            <xsl:with-param name="replacement" select="$replacement"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$outputString"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="lhs|rhs">
        <xsl:apply-templates select="funapp"/>
        <xsl:apply-templates select="var"/>
    </xsl:template>
        
    <xsl:template match="theory">
        (<xsl:value-of select="."/><xsl:text> </xsl:text>
        <xsl:for-each select="//theory[.=current()]">
            <xsl:sort select="parent::funcsym/name"/>
            <xsl:value-of select="parent::funcsym/name"/><xsl:if test="position()!=last()"><xsl:text> </xsl:text></xsl:if>
        </xsl:for-each>)<xsl:text> </xsl:text>
    </xsl:template>
    
    <xsl:template match="replacementmap">
        (<xsl:value-of select="parent::funcsym/name"/><xsl:text> </xsl:text><xsl:apply-templates select="entry"/>)<xsl:text> </xsl:text>
    </xsl:template>
    
    <xsl:template match="entry">
        <xsl:value-of select="."/>
        <xsl:if test="position()!=last()"><xsl:text> </xsl:text></xsl:if>
    </xsl:template>
</xsl:stylesheet>
